def corr_filter(corr_matrix, target_feature, abs_thresh=0.3):
    res = {}
    series = corr_matrix[target_feature]
    for idx, val in series.items():
        if abs(val) >= abs_thresh:
            res[idx] = val  # Feature names and corr coeff
    return res


from pianko.build_pipe import build_pipe
from sklearn.model_selection import cross_val_score
from numpy import mean

def first_tune(models, scoring, X_train, Y_train, transformers):
    res = dict()
    for name, model in models:
        model_pipeline = build_pipe(model, **transformers)
        cv_scores = cross_val_score(model_pipeline, X_train, Y_train, scoring=scoring)
        res[name] = cv_scores

    return dict(sorted(res.items(), reverse=True, key=lambda x: mean(x[1])))


from pianko import nan_statistics
from pianko import corr_filter
from pianko import build_pipe
from pianko import first_tune
from pianko import fine_tune

from pianko.plotting import plot_learning_curve

from pianko.transformers import CatEncoder
from pianko.transformers import ColumnKeeper
from pianko.transformers import ColumnRemover

from pianko.transformers import NanNumFiller
from pianko.transformers import NanCatFiller
from pianko.transformers import NanRemover
from pianko.transformers import LogTransformer
from pianko.transformers import QuantileRemover
from pianko.transformers import IQRRemover

from pianko.FeatureTransformer import FeaturesTransformers



import matplotlib.pyplot as plt
from sklearn.model_selection import learning_curve

def plot_learning_curve(model, X, y, train_sizes, cv, scoring):
    train_sizes, train_scores, validation_scores = learning_curve(
        estimator = model,
        X = X,
        y = y,
        train_sizes = train_sizes,
        cv = cv,
        scoring = scoring
    )
    plt.figure(figsize=(10, 10))
    plt.plot(train_sizes, train_scores.mean(axis=1), label='train err')
    plt.plot(train_sizes, validation_scores.mean(axis=1), label='val error')
    plt.ylabel('Error')
    plt.xlabel('X train size')
    plt.title('Learning curve')
    plt.legend()


from abc import abstractmethod
import numpy as np

class Transformer:
    def __init__(self):
        pass

    @abstractmethod
    def transform(self):
        pass

class CatEncoder(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, colname, dictionary):
        # dict with cat_feature_name: new_value
        # dict = {'FeatureA': 1} // encoded value
        series = df[colname]  # Series
        for f_name, new_val in dictionary.items():
            series[series == f_name] = new_val
        df[colname] = series

class ColumnKeeper(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, features_to_keep):
        df.drop(df.columns.difference(features_to_keep), 1, inplace=True)

class ColumnRemover(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, columns_to_remove):
        df.drop(columns=columns_to_remove, inplace=True)

class IQRRemover(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, columns, range_width=1.5):
        for c in columns:
            Q1 = df[c].quantile(0.25)
            Q3 = df[c].quantile(0.75)
            IQR = Q3 - Q1
            df = df[(df[c] < Q1 - range_width * IQR) &
                    (df[c] > Q3 + range_width * IQR)]
        return df

class LogTransformer(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, colnames, offset=0.001):
        for c in colnames:
            df[c] = df[c].map(lambda i: np.log(i) if i > 0 else np.log(i + offset))

class NanCatFiller(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, cols, val):
        for c in cols:
            df[c].fillna(value=val, inplace=True)

class NanNumFiller(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, cols, method):
        if method == 'median':
            for c in cols:
                df[c].fillna(value=df[c].median(), inplace=True)

class NanRemover(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, cols):
        df.drop(columns=cols, inplace=True)

class QuantileRemover(Transformer):
    def __init__(self):
        super().__init__()

    def transform(self, df, cols, percent_low, percent_high):
        for c in cols:
            q_low = df[c].quantile(percent_low)
            q_high = df[c].quantile(percent_high)
            df = df[df[c].between(q_low, q_high)]
            # return df[(df[c] > q_low) & (df[c] < q_high)]
        return df


from pianko.build_pipe import build_pipe
from sklearn.model_selection import GridSearchCV, RandomizedSearchCV

def fine_tune(models, X_train, Y_train, strategy, search_params, transformers):
    fine_tune_res = {}

    for name, model, param_grid in models:
        print(search_params)
        pipe = build_pipe(model, **transformers)

        if strategy == 'GridSearchCV':
            search_res = GridSearchCV(
                estimator=pipe,
                param_grid=param_grid,
                **search_params
            )

        elif strategy == 'RandomizedSearchCV':
            search_res = RandomizedSearchCV(
                pipe,
                param_grid,
                **search_params
            )
        search_res.fit(X_train, Y_train)
        fine_tune_res[name] = search_res

    return dict(sorted(fine_tune_res.items(), reverse=True, key=lambda x: x[1].best_score_))


import pandas as pd
import numpy as np

def nan_percentage(df, colname):
    return (df[colname].isnull().sum() / df.shape[0]) * 100

def nan_statistics(df, nan_thresh=0.0):
    res = {}
    nan_cols = df.loc[:, df.isna().any()].columns
    for col in nan_cols:
        res[col] = nan_percentage(df, col)
    print(f'Col -- Nan percentage')
    for key, val in sorted(res.items(), key=lambda item: item[1], reverse=True):
        if val >= nan_thresh * 100:
            print(key, val)
        else:
            del res[key]
    return res


from sklearn.compose import ColumnTransformer
from sklearn.pipeline import Pipeline

def build_pipe(model, **transformers):
    res_transformers = []
    for key, val in transformers.items():
        res_transformers.append((key, val.transformers, val.features))

    preprocess = ColumnTransformer(
        res_transformers
    )

    pipe = Pipeline(
        steps=[
            ('preprocessing', preprocess),
            ('model', model)
        ]
    )
    return pipe


from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.11'
DESCRIPTION = 'Package for df transformations using pandas'
LONG_DESCRIPTION = 'A package that contains transformers for NAN cleaning, IQR filtering, plotting learning curve.' \
                   'To avoid redundant code in the colab notebook'

# Setting up
setup(
    name="pianko",
    version=VERSION,
    author="Pavel Ianko",
    author_email="<pavel.evgenievich.ianko@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['numpy', 'pandas', 'sklearn', 'matplotlib'],
    keywords=['python', 'pandas', 'numpy'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
